@@ -1,11 +1,11 @@
 {-# LANGUAGE CPP, OverloadedStrings, RankNTypes, ScopedTypeVariables, StandaloneDeriving #-}
 {-# OPTIONS_GHC -Wall -fno-warn-orphans #-}
-module Main
-    ( tests
-    , main
-    ) where
+module Main where
 
+#if !MIN_VERSION_base(4,8,0)
 import Control.Applicative ((<$>))
+import Data.Monoid (mconcat, mempty)
+#endif
 import Debian.Debianize.Optparse(_flags, parseProgramArguments)
 import Control.Lens
 import Data.Algorithm.DiffContext (getContextDiff, prettyContextDiff)
@@ -15,42 +15,46 @@ import Data.List (sortBy)
 import Data.Map as Map (differenceWithKey, insert, intersectionWithKey)
 import qualified Data.Map as Map (elems, Map, toList)
 import Data.Maybe (fromMaybe)
-import Data.Monoid ((<>), mconcat, mempty)
+import Data.Monoid ((<>))
 import Data.Set as Set (fromList, union, insert)
 import Data.Text as Text (intercalate, split, Text, unlines, unpack)
 import Data.Version (Version(Version))
 import Debian.Changes (ChangeLog(..), ChangeLogEntry(..), parseEntry)
 import Debian.Debianize.BasicInfo (compilerFlavor, Flags, verbosity)
+import qualified Debian.Debianize.Bundled as Bundled (tests)
 import qualified Debian.Debianize.BinaryDebDescription as B
-import Debian.Debianize.CabalInfo as A
+import Debian.Debianize.CabalInfo as A (CabalInfo, debInfo, epochMap, newCabalInfo)
 import Debian.Debianize.CopyrightDescription
 import Debian.Debianize.DebianName (mapCabal, splitCabal)
 import qualified Debian.Debianize.DebInfo as D
+import Debian.Debianize.Details (debianDefaults)
 import Debian.Debianize.Files (debianizationFileMap)
 import Debian.Debianize.Finalize (debianize {-, finalizeDebianization-})
 import Debian.Debianize.Goodies (doBackups, doExecutable, doServer, doWebsite, tightDependencyFixup)
 import Debian.Debianize.InputDebian (inputDebianization)
 import Debian.Debianize.Monad (CabalT, evalCabalT, execCabalM, execCabalT, liftCabal, execDebianT, DebianT, evalDebianT)
+import Debian.Debianize.Output (performDebianization)
 import Debian.Debianize.Prelude (withCurrentDirectory)
 import qualified Debian.Debianize.SourceDebDescription as S
 import Debian.Debianize.VersionSplits (DebBase(DebBase))
+import Debian.GHC (hvrCompilerPATH, withModifiedPATH)
 import Debian.Pretty (ppShow)
-import Debian.Policy (databaseDirectory, PackageArchitectures(All), PackagePriority(Extra), parseMaintainer, Section(MainSection), SourceFormat(Native3), StandardsVersion(..), getDebhelperCompatLevel, getDebianStandardsVersion, License(..))
+import Debian.Policy (databaseDirectory, PackageArchitectures(All), PackagePriority(Extra), parseMaintainer, Section(MainSection), SourceFormat(Native3), StandardsVersion(..) {-, getDebhelperCompatLevel, getDebianStandardsVersion, License(..)-})
 import Debian.Relation (BinPkgName(..), Relation(..), SrcPkgName(..), VersionReq(..))
 import Debian.Release (ReleaseName(ReleaseName, relName))
-import Debian.Version (parseDebianVersion', buildDebianVersion)
-import Distribution.Compiler (CompilerFlavor(GHC))
+import Debian.Version (parseDebianVersion'{-, buildDebianVersion-})
+import Distribution.Compiler (CompilerFlavor({-GHC,-} GHCJS))
 import Distribution.Package (PackageName(PackageName))
 import Prelude hiding (log)
 import System.Directory (doesDirectoryExist)
 import System.Exit (exitWith, ExitCode(..))
 import System.FilePath ((</>))
+import System.Git (gitResetSubdir)
 import System.Process (readProcessWithExitCode)
 import Test.HUnit
 import Text.ParserCombinators.Parsec.Rfc2822 (NameAddr(..))
 import Text.PrettyPrint.HughesPJClass (pPrint, text, Doc)
 
-
 -- | Backward compatibility. Should be fixed.
 newFlags :: IO Flags
 newFlags = _flags <$> parseProgramArguments
@@ -68,10 +72,11 @@ defaultAtoms =
 
 -- | Force the compiler version to 7.6 to get predictable outputs
 testAtoms :: IO CabalInfo
-testAtoms = newFlags >>= newCabalInfo >>= return . ghc763
+testAtoms = ghc763 <$> (newFlags >>= newCabalInfo)
     where
-      ghc763 :: CabalInfo -> CabalInfo
-      ghc763 atoms = set (A.debInfo . D.flags . compilerFlavor) GHC atoms
+      ghc763 :: Either String CabalInfo -> CabalInfo
+      ghc763 (Left s) = error $ "testAtoms - failed to build CabalInfo: " ++ s
+      ghc763 (Right atoms) = {-set (A.debInfo . D.flags . compilerFlavor) GHC-} atoms
 
 -- | Create a Debianization based on a changelog entry and a license
 -- value.  Uses the currently installed versions of debhelper and
@@ -92,7 +97,8 @@ newDebianization' level standards =
        (A.debInfo . D.control . S.standardsVersion) .= standards
 
 tests :: Test
-tests = TestLabel "Debianization Tests" (TestList [-- 1 and 2 do not input a cabal package - we're not ready to
+tests = TestLabel "Debianization Tests" (TestList [Bundled.tests,
+                                                   -- 1 and 2 do not input a cabal package - we're not ready to
                                                    -- debianize without a cabal package.
                                                    {- test1 "test1",
                                                    test2 "test2", -}
@@ -100,10 +106,14 @@ tests = TestLabel "Debianization Tests" (TestList [-- 1 and 2 do not input a cab
                                                    test4 "test4 - test-data/clckwrks-dot-com",
                                                    test5 "test5 - test-data/creativeprompts",
                                                    test6 "test6 - test-data/artvaluereport2",
-                                                   test7 "test7 - debian/Debianize.hs",
+                                                   -- test7 "test7 - debian/Debianize.hs",
                                                    test8 "test8 - test-data/artvaluereport-data",
                                                    test9 "test9 - test-data/alex",
-                                                   test10 "test10 - test-data/archive" {- ,
+                                                   test10 "test10 - test-data/archive" {-,
+                                                   -- This works, but it adds a dependency on ghcjs to the test suite
+                                                   -- test11 "test11 - test-data/diff",
+                                                   -- And this requires ghc-8.0.1 from the hvr compiler repo
+                                                   -- test12 "test12 - test-data/diff",
                                                    issue23 "issue23" -}])
 
 issue23 :: String -> Test
@@ -595,8 +605,8 @@ test6 :: String -> Test
 test6 label =
     TestLabel label $
     TestCase (do dist <- findBuildDir
-                 (code, out, _err) <- readProcessWithExitCode "runhaskell" ["--ghc-arg=-package-db=" ++ dist ++ "/package.conf.inplace", "test-data/artvaluereport2/input/debian/Debianize.hs", "--dry-run", "--verbose"] ""
-                 assertEqual label (ExitSuccess, "") (code, out))
+                 (code, out, err) <- readProcessWithExitCode "runhaskell" ["--ghc-arg=-package-db=" ++ dist ++ "/package.conf.inplace", "test-data/artvaluereport2/input/debian/Debianize.hs", "--dry-run", "--verbose"] ""
+                 assertEqual label (ExitSuccess, "", err) (code, out, err))
 
 test7 :: String -> Test
 test7 label =
@@ -615,7 +625,9 @@ test8 label =
                       outTop = "test-data/artvaluereport-data/output"
                   (old :: D.DebInfo) <- withCurrentDirectory outTop $ newFlags >>= execDebianT inputDebianization . D.makeDebInfo
                   let log = view D.changelog old
-                  new <- withCurrentDirectory inTop $ newFlags >>= newCabalInfo >>= execCabalT (debianize (defaultAtoms >> customize log))
+                  new <- withCurrentDirectory inTop $
+                         newFlags >>= newCabalInfo >>=
+                         either (error "test8 - newCabalInfo failed") (execCabalT (debianize (defaultAtoms >> customize log)))
                   diff <- diffDebianizations old (view debInfo new)
                   assertEmptyDiff label diff
              )
@@ -634,7 +646,9 @@ test9 label =
     TestLabel label $
     TestCase (do let inTop = "test-data/alex/input"
                      outTop = "test-data/alex/output"
-                 new <- withCurrentDirectory inTop $ newFlags >>= newCabalInfo >>= execCabalT (debianize (defaultAtoms >> customize))
+                 new <- withCurrentDirectory inTop $
+                        newFlags >>= newCabalInfo >>=
+                        either (error "test9 - newCabalInfo failed") (execCabalT (debianize (defaultAtoms >> customize)))
                  let Just (ChangeLog (entry : _)) = view (debInfo . D.changelog) new
                  old <- withCurrentDirectory outTop $ newFlags >>= execDebianT (inputDebianization >> copyChangelogDate (logDate entry)) . D.makeDebInfo
                  diff <- diffDebianizations old (view debInfo new)
@@ -674,7 +688,9 @@ test10 label =
                      outTop = "test-data/archive/output"
                  old <- withCurrentDirectory outTop $ newFlags >>= execDebianT inputDebianization . D.makeDebInfo
                  let Just (ChangeLog (entry : _)) = view D.changelog old
-                 new <- withCurrentDirectory inTop $ newFlags >>= newCabalInfo >>= execCabalT (debianize (defaultAtoms >> customize >> (liftCabal $ copyChangelogDate $ logDate entry)))
+                 new <- withCurrentDirectory inTop $
+                        newFlags >>= newCabalInfo >>=
+                        either (error "test10 - newCabalInfo failed") (execCabalT (debianize (defaultAtoms >> customize >> (liftCabal $ copyChangelogDate $ logDate entry))))
                  diff <- diffDebianizations old (view debInfo new)
                  assertEmptyDiff label diff)
     where
@@ -737,10 +753,35 @@ diffDebianizations old new =
 sortBinaryDebs :: DebianT IO ()
 sortBinaryDebs = (D.control . S.binaryPackages) %= sortBy (compare `on` view B.package)
 
+test11 :: String -> Test
+test11 label =
+    TestLabel label $
+    TestCase $ do let input = "test-data/diff/input"
+                      expected = "test-data/diff/expected.ghcjs"
+                  gitResetSubdir input
+                  withCurrentDirectory input (performDebianization (do debianDefaults
+                                                                       (debInfo . D.flags . compilerFlavor) .= GHCJS
+                                                                       (debInfo . D.testsStatus) .= D.TestsDisable
+                                                                       (debInfo . D.sourceFormat) .= Native3))
+                  assertEmptyDiff expected input
+
+test12 :: String -> Test
+test12 label =
+    TestLabel label $
+    TestCase $ withModifiedPATH (hvrCompilerPATH (Version [8,0,1] [])) $
+               do let input = "test-data/diff/input"
+                      expected = "test-data/diff/expected.hvr"
+                  gitResetSubdir input
+                  withCurrentDirectory input (performDebianization (do debianDefaults
+                                                                       (debInfo . D.testsStatus) .= D.TestsDisable
+                                                                       (debInfo . D.sourceFormat) .= Native3))
+                  assertEmptyDiff expected input
+
+
 main :: IO ()
 main = do
- counts <- runTestTT tests
- exitWith $ if errors counts + failures counts > 0
+ cts <- withModifiedPATH (const "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/usr/local/games") (runTestTT tests)
+ exitWith $ if errors cts + failures cts > 0
             then ExitFailure 1
             else ExitSuccess
 
@@ -749,7 +790,7 @@ main = do
 assertEmptyDiff :: String -- ^ The message prefix
                 -> String -- ^ The actual diff
                 -> Assertion
-assertEmptyDiff preface "" = return ()
+assertEmptyDiff _preface "" = return ()
 assertEmptyDiff preface diff = assertFailure $
  (if null preface then "" else preface ++ "\n") ++
  "diff not empty:\n" ++
