@@ -40,13 +40,22 @@ import Debian.Policy
 import Debian.Relation
 import Debian.Version (DebianVersion, parseDebianVersion')
 import Distribution.Compiler (CompilerFlavor(..))
+#if MIN_VERSION_Cabal(2,0,0)
+import Distribution.Package (PackageName, mkPackageName, unPackageName)
+import Distribution.PackageDescription (FlagName, mkFlagName)
+#else
 import Distribution.Package (PackageName(..))
 import Distribution.PackageDescription (FlagName(FlagName))
+#endif
 import GHC.Generics
 import System.Environment (getArgs)
 import System.FilePath(splitFileName, (</>))
 import System.Process (showCommandForUser)
+#if MIN_VERSION_hsemail(2,0,0)
+import Text.Parsec.Rfc2822 (NameAddr(..))
+#else
 import Text.ParserCombinators.Parsec.Rfc2822 (NameAddr(..))
+#endif
 import Text.PrettyPrint.ANSI.Leijen (linebreak, (<+>), string, indent)
 import qualified  Debian.Debianize.DebInfo as D
 import qualified Data.Map as Map
@@ -176,7 +185,7 @@ nameAddrR :: O.ReadM NameAddr
 nameAddrR = either fail return =<< parseMaintainer <$> O.str
 
 relationsR :: O.ReadM Relations
-relationsR = either (fail . show) return =<< parseRelations <$> O.str
+relationsR = either (fail . show) return =<< parseRelations <$> (O.str :: O.ReadM String)
 
 mappingR :: O.ReadM (String, Relations)
 mappingR = span (/= ':') <$> O.str >>= \case
@@ -196,15 +205,28 @@ extraRelationsR :: O.ReadM (BinPkgName, Relations)
 extraRelationsR = first BinPkgName <$> mappingR
 
 cabalDebMappingR :: O.ReadM CabalDebMapping
+#if MIN_VERSION_Cabal(2,0,0)
+cabalDebMappingR = CabalDebMapping . first mkPackageName <$> mappingR
+#else
 cabalDebMappingR = CabalDebMapping . first PackageName <$> mappingR
+#endif
 
 cabalEpochMappingR :: O.ReadM CabalEpochMapping
+#if MIN_VERSION_Cabal(2,0,0)
+cabalEpochMappingR = CabalEpochMapping . first mkPackageName <$> epochMappingR
+#else
 cabalEpochMappingR = CabalEpochMapping . first PackageName <$> epochMappingR
+#endif
 
 cabalFlagMappingR :: O.ReadM CabalFlagMapping
 cabalFlagMappingR = O.str >>= \case
+#if MIN_VERSION_Cabal(2,0,0)
+  ('-' : str) -> return $ CabalFlagMapping (mkFlagName str, False)
+  str -> return $ CabalFlagMapping (mkFlagName str, True)
+#else
   ('-' : str) -> return $ CabalFlagMapping (FlagName str, False)
   str -> return $ CabalFlagMapping (FlagName str, True)
+#endif
 
 -- Here are parser for BehaviorAdjustment and next are parsers for
 -- every field of this data.  Please, keep parsers declarations in
@@ -309,7 +331,7 @@ debianNameBaseP = O.option (Just . DebBase <$> O.str) m where
     ]
 
 debianVersionP :: O.Parser (Maybe DebianVersion)
-debianVersionP = O.option (Just . parseDebianVersion' <$> O.str) m where
+debianVersionP = O.option (Just . parseDebianVersion' <$> (O.str :: O.ReadM String)) m where
   m = O.help helpMsg
       <> O.long "deb-version"
       <> O.metavar "DEBIANVERSION"
@@ -634,8 +656,13 @@ handleBehaviorAdjustment (BehaviorAdjustment {..}) = do
   D.debVersion .= _debianVersion
   D.uploadersOption %= (++ _uploaders)
   D.extraDevDeps %= (++ concatMap unpack _devDep)
+#if MIN_VERSION_Cabal(2,0,0)
+  forM_ _cabalDebMapping $ \(CabalDebMapping (pkg, rels)) -> do
+    D.extraLibMap %= Map.insert (unPackageName pkg) rels
+#else
   forM_ _cabalDebMapping $ \(CabalDebMapping (PackageName pkg, rels)) -> do
     D.extraLibMap %= Map.insert pkg rels
+#endif
   addExtra _extraDepends B.depends
   addExtra _extraConflicts B.conflicts
   addExtra _extraProvides B.provides
