@@ -44,7 +44,7 @@ import System.FilePath ((</>), takeDirectory)
 import System.IO (hPutStrLn, stderr)
 import System.Process (readProcessWithExitCode, showCommandForUser)
 import Text.PrettyPrint.HughesPJClass (text)
-import System.Posix.Env (setEnv)
+-- import System.Posix.Env (setEnv)
 
 -- | Run the script in @debian/Debianize.hs@ with the given command
 -- line arguments.  Returns @True@ if the script exists and succeeds.
@@ -75,12 +75,20 @@ runDebianizeScript args =
 -- this function does everything else.
 performDebianization :: CabalT IO () -> IO ()
 performDebianization custom =
-  parseProgramArguments >>= \CommandLineOptions {..} ->
-    newCabalInfo _flags >>= (evalCabalT $ do
+  parseProgramArguments >>= \CommandLineOptions {..} -> do
+    -- _ <- try (readProcessWithExitCode "apt-get" ["install", "-y", "--force-yes", hcDeb (view compilerFlavor _flags)] "")
+    newCabalInfo _flags >>= either
+                               (error . ("peformDebianization - " ++))
+                               (evalCabalT $ do
                                 handleBehaviorAdjustment _adjustment
                                 debianize custom
                                 finishDebianization)
 
+-- hcDeb :: CompilerFlavor -> String
+-- hcDeb GHC = "ghc"
+-- hcDeb GHCJS = "ghcjs"
+-- hcDeb flavor = error $ "hcDeb - unexpected CompilerFlavor: " ++ show flavor
+
 -- | Depending on the options in @atoms@, either validate, describe,
 -- or write the generated debianization.
 finishDebianization :: forall m. (MonadIO m, Functor m) => StateT CabalInfo m ()
