@@ -64,7 +64,11 @@ import System.Environment (getEnvironment)
 import System.FilePath ((</>))
 import System.Process (readProcess)
 import Text.Parsec (parse)
+#if MIN_VERSION_hsemail(2,0,0)
+import Text.Parsec.Rfc2822 (address, NameAddr(..))
+#else
 import Text.ParserCombinators.Parsec.Rfc2822 (address, NameAddr(..))
+#endif
 import Text.PrettyPrint.HughesPJClass (Pretty(pPrint), text)
 import Text.Read (readMaybe)
 
@@ -122,7 +126,7 @@ debianPackageVersion name =
 -- installed.
 getDebhelperCompatLevel :: IO (Maybe Int)
 getDebhelperCompatLevel =
-    debianPackageVersion "debhelper" >>= return . fmap (read . takeWhile (/= '.') . version)
+    debianPackageVersion "debhelper" >>= return . fmap (min 9 . read . takeWhile (/= '.') . version)
 
 data StandardsVersion = StandardsVersion Int Int Int (Maybe Int) deriving (Eq, Ord, Show, Data, Typeable)
 
